==> file4life-1.1.5-source.tgz: file4life-1.1.5/src/file4life.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module Main where

import F4L
-- import F4L.Record( initialOpEnv )
import F4L.Actions
import F4L.BufferMonad
import F4L.FSMonad( FSMonad, DelFSMonad, ParFSMonad, deLoggingIO )
import F4L.Interactive( runShell )
import F4L.Knowledge
import F4L.Test( runTest )
import F4L.Utils( readN )

-- import Control.Monad
-- import Control.Monad.State
import Control.Monad.State.Strict
import Data.List( foldl', intercalate, isPrefixOf )
import Data.Maybe( fromMaybe )
import System.Console.GetOpt
import System.Console.Haskeline.MonadException
import System.Directory( getAppUserDataDirectory )
import System.Environment( getArgs )
import System.FilePath( combine, joinPath )


programVersion = "1.1.5"


{---------------------------------------- Running ----------------------------------------}

-- | Configuration object
data RunConf = RunConf { rc_cmd :: RunCommand
                       , rc_exif :: Bool
                       , rc_rescan :: Bool
                       , rc_tags :: [String]
                       , rc_priority :: Maybe Priority
                       , rc_maxsize :: Maybe Int
                       , rc_db :: FilePath
                       , rc_verbosity :: Int
                       , rc_log :: Bool
                       }

data RunCommand = Usage
                | ScanFiles
                | Dump
                | Facts
                | ShowCoverage
                | ShowStats
                | ShowStorages
                | ShowSessions
                | SliceSessions
                | Merge
                | Query
                | Backup FilePath
                | Restore
                | CheckBackup FilePath
                | Interact
                | MakeTest Int
                | Test
                | ListStorageContents String


defaultConf appDir = RunConf Usage True False [] Nothing Nothing ( combine appDir "db.f4l" ) 0 False

withCommand c rc = rc { rc_cmd = c }
withExif x rc = rc { rc_exif = x }
withRescan r rc = rc { rc_rescan = r }
withTag t rc = rc { rc_tags = rc_tags rc ++ [t] }
withPriority p rc = rc { rc_priority = Just ( Priority $ read p ) }
withMaxsize s rc = rc { rc_maxsize = Just s }
withDb db rc = rc { rc_db = db }
withLog log rc = rc { rc_log = log }
moreVerbose rc = rc { rc_verbosity = rc_verbosity rc + 1 }
withShowCommand "coverage" rc = rc { rc_cmd = ShowCoverage }
withShowCommand "stats" rc = rc { rc_cmd = ShowStats }
withShowCommand "storages" rc = rc { rc_cmd = ShowStorages }
withShowCommand "sessions" rc = rc { rc_cmd = ShowSessions }
withShowCommand x _ = error $ "Unknown show option: " ++ x
withListStorageCommand sid rc = rc { rc_cmd = ListStorageContents sid }


options = [
          -- options
            Option "" [ "noexif" ] ( NoArg $ withExif False ) "no exif"
          , Option "R" [ "rescan" ] ( NoArg $ withRescan True ) "re-scan known files"
          , Option "p" [ "priority" ] ( ReqArg withPriority "priority (1 lowest - 10 highest)" ) "set priority"
          , Option "S" [ "size" ] ( ReqArg ( withMaxsize . readN ) "size" ) "max size"
          , Option ""  [ "db" ] ( ReqArg withDb "db file" ) "use db"
          , Option ""  [ "IOlog" ] ( NoArg $ withLog True ) "log IO operations to file"

          -- commands
          , Option "h" [ "help" ] ( NoArg $ withCommand Usage ) "show usage"
          , Option "a" [ "scan" ] ( NoArg $ withCommand ScanFiles ) "scan files"          
          , Option "d" [ "dump" ] ( NoArg $ withCommand Dump ) "dump DB"
          , Option "f" [ "facts" ] ( NoArg $ withCommand Facts ) "dump facts"
          , Option "s" [ "show" ] ( ReqArg withShowCommand "coverage | sessions | stats | storages" ) "show info"
          , Option "m" [ "merge" ] ( NoArg $ withCommand Merge ) "merge DBs"
          , Option ""  [ "slice" ] ( NoArg $ withCommand SliceSessions ) "slice sessions"
          , Option "L" [ "list" ] ( ReqArg withListStorageCommand "storage ID" ) "show stats"
          , Option "q" [ "query" ] ( NoArg $ withCommand Query ) "query"
          , Option "b" [ "backup" ] ( ReqArg ( withCommand . Backup ) "destination directory" ) "backup to directory"
          , Option ""  [ "check-backup", "cb" ] ( ReqArg ( withCommand . CheckBackup ) "backup directory" ) "check backed-up files in directory"
          , Option "r" [ "restore" ] ( NoArg $ withCommand Restore ) "restore"
          , Option "v" [ "verbose" ] ( NoArg moreVerbose ) "increase verbosity"
          , Option "i" [ "interact" ] ( NoArg $ withCommand Interact ) "interactive shell" 
          , Option "T" [ "test" ] ( NoArg $ withCommand Test ) "run tests"
          ]


main :: IO()
main = getArgs >>= runMain


noK = ()

fileK :: ( SeenStorages, KnownData )
fileK = undefined

statsK :: ( FactCounter, FileSet )
statsK = undefined

kd :: KnownData
kd = undefined

runMain :: [String] -> IO ()
runMain args = do
  -- Read options into conf
  appDir <- getAppUserDataDirectory "f4l"
  ( conf, paths ) <- case getOpt Permute options args of
    (o, ps, []) -> return (foldl' (flip id) (defaultConf appDir) o, ps)
    (_, _, errs) -> fail( concat errs ++ usageInfo "Usage" options )
  env <- forceGetEnvId appDir
  let
    defaultState :: Knowledge k => F4LState k ()
    defaultState = f4lState (MetaDB $ rc_db conf) (flip Verbosity True $ toEnum $ rc_verbosity conf) Ask (rc_exif conf) (rc_rescan conf) ()
  let
    inSession' :: ( FSMonad m, BufferMonad m, Knowledge k ) => F4LMonad k m a -> GenF4LMonad k s m a
    inSession' = inSession env
  let
    noExtra :: MonadIO m => m () -> m ()
    noExtra act = case paths of
      [] -> act
      _ -> liftIO $ putStrLn $ "Extra arguments: " ++ intercalate ", " paths
  let
    -- Execution can happen in different monads
    exec :: ( Monad m, MonadIO m, FSMonad m, BufferMonad m, DelFSMonad m, ParFSMonad m, MonadException m ) => m ()
    exec = case rc_cmd conf of
      Usage -> noExtra $ liftIO $ do
        putStrLn $ "file4life version " ++ programVersion
        putStrLn $ usageInfo "Usage" options
      Dump -> noExtra $ evalStateT ( withKnowledge noK >> onErrorDie ( fromDB dumpOps ) ) defaultState
      MakeTest i -> noExtra $ evalStateT ( withKnowledge fileK >> ( inSession' $ addTestFiles i ) ) defaultState
      Facts -> noExtra $ evalStateT ( withKnowledge noK >> onErrorDie ( fromDB dumpFacts ) ) defaultState
      ShowCoverage -> noExtra $ do
        s <- evalStateT ( withKnowledge kd >> onErrorDie loadCached >> coverage ) defaultState
        mapM_ ( liftIO . print ) s
      ShowStats -> noExtra $ do
        s <- evalStateT ( withKnowledge statsK >> onErrorDie loadCached >> stats ) defaultState
        mapM_ ( \r -> liftIO $ putStrLn( fst r ++ ": " ++ snd r ) ) s
      ShowStorages -> noExtra $ do
        s <- evalStateT ( withKnowledge ( noKnowledge :: SeenStorages ) >> onErrorDie loadCached >> storages ) defaultState
        mapM_ ( \r -> liftIO $ putStrLn( fst r ++ ": " ++ show ( snd r ) ) ) s
      ShowSessions ->
        -- liftIO $ onErrorDie ( fromDB $ return . sessions ) >>= mapM_ ( putStrLn . formatSession . fst )
        noExtra $ liftIO $ evalStateT ( withKnowledge noK >> ( onErrorDie ( fromDB $ return . sessions ) ) ) defaultState >>= mapM_ ( putStrLn . formatSession . fst )
      SliceSessions ->
        case reverse paths of
          [] -> error "Arguments: session1 ... outputFile"
          ofile : ids -> do
            importExportSessions ( \ s -> any ( `isPrefixOf` sdat_id s ) ids ) ( rc_db conf ) ofile defaultState
      ListStorageContents sid -> noExtra $ do
        s <- evalStateT ( listStorageContents sid ) defaultState
        mapM_ ( \r -> liftIO $ putStrLn( joinPath ( fst r ) ++ ": " ++ show ( snd r ) ) ) s
      ScanFiles -> do
        execStateT ( withKnowledge fileK >> ( onErrorDie initDB >> onErrorDie loadCached >> inSession' ( addFiles (rc_priority conf) paths ) ) ) defaultState
        return ()
      Query -> error "Not implemented"
      Backup dest ->
        flip evalStateT defaultState $ do
          onErrorDie loadCached
          inSession' ( withKnowledge kd >> ( autoBackup dest $ fromMaybe 1000000 $ rc_maxsize conf ) )
      Restore ->
        case paths of
          [sid, src, dest] -> evalStateT ( withKnowledge kd >> ( inSession' $ onErrorDie $ restore True sid src dest ) >> return () ) defaultState
          _ -> error "Usage: restore srcstor srcpath destpath"
      Merge ->
        case reverse paths of
          dest : srcs -> mergeDBs ( reverse srcs ) dest defaultState
          _ -> error "Usage: merge DBfile ... destDB"
      CheckBackup b -> noExtra $ do
        checkBackup ( not $ rc_rescan conf ) b
      Interact ->
        noExtra $ evalStateT ( onErrorDie initDB >> onErrorDie loadCached >> runShell env ) defaultState
      Test -> liftIO $ runTest paths

  -- Execute action in appropriate monad
  case rc_log conf of
    True -> flip evalStateT Nothing $ deLoggingIO $ runIOWithBuffer exec
    False -> runIOWithBuffer exec

==> file4life-1.2.0-source.tgz: file4life-1.2.0/src/file4life.hs <==
{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014-2017
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module Main where

import F4L
-- import F4L.Record( initialOpEnv )
import F4L.Actions
import F4L.BufferMonad
import F4L.FSMonad( FSMonad, DelFSMonad, ParFSMonad, deLoggingIO )
import F4L.Interactive( runShell )
import F4L.Knowledge
import F4L.Test( runTest )
import F4L.Utils( readN )

-- import Control.Monad
-- import Control.Monad.State
import Control.Monad.State.Strict
import Data.ByteString( pack )
import Data.List( foldl', intercalate, isPrefixOf )
import Data.Maybe( fromMaybe )
import System.Console.GetOpt
import System.Console.Haskeline.MonadException
import System.Directory( getAppUserDataDirectory )
import System.Environment( getArgs )
import System.FilePath( combine, joinPath )
import System.Remote.Monitoring


programVersion = "1.2.0"


{---------------------------------------- Running ----------------------------------------}

-- | Configuration object
data RunConf = RunConf { rc_cmd :: RunCommand
                       , rc_exif :: Bool
                       , rc_rescan :: Bool
                       , rc_tags :: [String]
                       , rc_priority :: Maybe Priority
                       , rc_maxsize :: Maybe Int
                       , rc_db :: FilePath
                       , rc_verbosity :: Int
                       , rc_log :: Bool
                       , rc_ekg :: Maybe Int
                       }

data RunCommand = Usage
                | ScanFiles
                | Dump
                | Facts
                | ShowCoverage
                | ShowStats
                | ShowStorages
                | ShowSessions
                | SliceSessions
                | TruncateDB
                | Merge
                | Query
                | Backup FilePath
                | Restore
                | CheckBackup FilePath
                | Interact
                | MakeTest Int
                | Test
                | ListStorageContents String


defaultConf appDir = RunConf Usage True False [] Nothing Nothing ( combine appDir "db.f4l" ) 0 False Nothing


withCommand c rc = rc { rc_cmd = c }
withExif x rc = rc { rc_exif = x }
withRescan r rc = rc { rc_rescan = r }
withTag t rc = rc { rc_tags = rc_tags rc ++ [t] }
withPriority p rc = rc { rc_priority = Just ( Priority $ read p ) }
withMaxsize s rc = rc { rc_maxsize = Just s }
withDb db rc = rc { rc_db = db }
withLog log rc = rc { rc_log = log }
withEkg :: String -> RunConf -> RunConf
withEkg ekg_port rc = rc { rc_ekg = Just $ read ekg_port }
moreVerbose rc = rc { rc_verbosity = rc_verbosity rc + 1 }
withShowCommand "coverage" rc = rc { rc_cmd = ShowCoverage }
withShowCommand "stats" rc = rc { rc_cmd = ShowStats }
withShowCommand "storages" rc = rc { rc_cmd = ShowStorages }
withShowCommand "sessions" rc = rc { rc_cmd = ShowSessions }
withShowCommand x _ = error $ "Unknown show option: " ++ x
withListStorageCommand sid rc = rc { rc_cmd = ListStorageContents sid }


options = [
          -- options
            Option "" [ "noexif" ] ( NoArg $ withExif False ) "no exif"
          , Option "R" [ "rescan" ] ( NoArg $ withRescan True ) "re-scan known files"
          , Option "p" [ "priority" ] ( ReqArg withPriority "priority (1 lowest - 10 highest)" ) "set priority"
          , Option "S" [ "size" ] ( ReqArg ( withMaxsize . readN ) "size" ) "max size"
          , Option ""  [ "db" ] ( ReqArg withDb "db file" ) "use db"
          , Option ""  [ "IOlog" ] ( NoArg $ withLog True ) "log IO operations to file"
          , Option ""  [ "ekg" ] ( OptArg ( withEkg . fromMaybe "8000" ) "port (8000)" ) "start EKG monitoring"

          -- commands
          , Option "h" [ "help" ] ( NoArg $ withCommand Usage ) "show usage"
          , Option "a" [ "scan" ] ( NoArg $ withCommand ScanFiles ) "scan files"          
          , Option "d" [ "dump" ] ( NoArg $ withCommand Dump ) "dump DB"
          , Option "f" [ "facts" ] ( NoArg $ withCommand Facts ) "dump facts"
          , Option "s" [ "show" ] ( ReqArg withShowCommand "coverage | sessions | stats | storages" ) "show info"
          , Option "m" [ "merge" ] ( NoArg $ withCommand Merge ) "merge DBs"
          , Option ""  [ "slice" ] ( NoArg $ withCommand SliceSessions ) "slice sessions"
          , Option ""  [ "truncate" ] ( NoArg $ withCommand TruncateDB ) "truncate DB"
          , Option "L" [ "list" ] ( ReqArg withListStorageCommand "storage ID" ) "show stats"
          , Option "q" [ "query" ] ( NoArg $ withCommand Query ) "query"
          , Option "b" [ "backup" ] ( ReqArg ( withCommand . Backup ) "destination directory" ) "backup to directory"
          , Option ""  [ "check-backup", "cb" ] ( ReqArg ( withCommand . CheckBackup ) "backup directory" ) "check backed-up files in directory"
          , Option "r" [ "restore" ] ( NoArg $ withCommand Restore ) "restore"
          , Option "v" [ "verbose" ] ( NoArg moreVerbose ) "increase verbosity"
          , Option "i" [ "interact" ] ( NoArg $ withCommand Interact ) "interactive shell" 
          , Option "T" [ "test" ] ( NoArg $ withCommand Test ) "run tests"
          ]


main :: IO()
main = getArgs >>= runMain BinaryCache


noK = ()


fileK :: ( SeenStorages, KnownData )
fileK = undefined


statsK :: ( FactCounter, FileSet )
statsK = undefined


kd :: KnownData
kd = undefined


maxUncached = 100000


load ser = loadCached ser maxUncached


runMain :: CacheSerialization -> [String] -> IO ()
runMain ser args = do
  -- Read options into conf
  appDir <- getAppUserDataDirectory "f4l"
  ( conf, paths ) <- case getOpt Permute options args of
    (o, ps, []) -> return (foldl' (flip id) (defaultConf appDir) o, ps)
    (_, _, errs) -> fail( concat errs ++ usageInfo "Usage" options )
  flip ( maybe $ return () ) ( rc_ekg conf ) $ \ port -> do
    forkServer ( pack $ map ( toEnum . fromEnum ) "localhost" ) port
    putStrLn $ "Started server on http://localhost:" ++ show port
  env <- forceGetEnvId appDir
  let
    defaultState :: Knowledge k => F4LState k ()
    defaultState = f4lState (MetaDB $ rc_db conf) (flip Verbosity True $ toEnum $ rc_verbosity conf) Ask (rc_exif conf) (rc_rescan conf) ()
  let
    inSession' :: ( FSMonad m, BufferMonad m, Knowledge k ) => F4LMonad k m a -> GenF4LMonad k s m a
    inSession' = inSession env
  let
    noExtra :: MonadIO m => m () -> m ()
    noExtra act = case paths of
      [] -> act
      _ -> liftIO $ putStrLn $ "Extra arguments: " ++ intercalate ", " paths
  let
    -- Execution can happen in different monads
    exec :: ( Monad m, MonadIO m, FSMonad m, BufferMonad m, DelFSMonad m, ParFSMonad m, MonadException m ) => m ()
    exec = case rc_cmd conf of
      Usage -> noExtra $ liftIO $ do
        putStrLn $ "file4life version " ++ programVersion
        putStrLn $ usageInfo "Usage" options
      Dump -> noExtra $ evalStateT ( withKnowledge noK >> onErrorDie ( fromDB dumpOps ) ) defaultState
      MakeTest i -> noExtra $ evalStateT ( withKnowledge fileK >> ( inSession' $ addTestFiles i ) ) defaultState
      Facts -> noExtra $ evalStateT ( withKnowledge noK >> onErrorDie ( fromDB dumpFacts ) ) defaultState
      ShowCoverage -> noExtra $ do
        s <- evalStateT ( withKnowledge kd >> onErrorDie ( load ser ) >> coverage ) defaultState
        mapM_ ( liftIO . print ) s
      ShowStats -> noExtra $ do
        s <- evalStateT ( withKnowledge statsK >> onErrorDie ( load ser ) >> stats ) defaultState
        mapM_ ( \r -> liftIO $ putStrLn( fst r ++ ": " ++ snd r ) ) s
      ShowStorages -> noExtra $ do
        s <- evalStateT ( withKnowledge ( noKnowledge :: SeenStorages ) >> onErrorDie ( load ser ) >> storages ) defaultState
        mapM_ ( \r -> liftIO $ putStrLn( fst r ++ ": " ++ show ( snd r ) ) ) s
      ShowSessions ->
        -- liftIO $ onErrorDie ( fromDB $ return . sessions ) >>= mapM_ ( putStrLn . formatSession . fst )
        noExtra $ liftIO $ evalStateT ( withKnowledge noK >> ( onErrorDie ( fromDB $ return . sessions ) ) ) defaultState >>= mapM_ ( putStrLn . formatSession . fst )
      SliceSessions ->
        case reverse paths of
          [] -> error "Arguments: session1 ... outputFile"
          ofile : ids -> do
            importExportSessions ( \ s -> any ( `isPrefixOf` sdat_id s ) ids ) ( rc_db conf ) ofile defaultState
      TruncateDB ->
        case paths of
          [ n, ofile ] -> do
            truncateDB ( read n ) ( rc_db conf ) ofile
          [ n, ifile, ofile ] -> do
            truncateDB ( read n ) ifile ofile
          _ ->
            error "Arguments: n [ inputFile ] outputFile"
      ListStorageContents sid -> noExtra $ do
        s <- evalStateT ( listStorageContents sid ) defaultState
        mapM_ ( \r -> liftIO $ putStrLn( joinPath ( fst r ) ++ ": " ++ show ( snd r ) ) ) s
      ScanFiles -> do
        execStateT ( withKnowledge fileK >> ( onErrorDie initDB >> onErrorDie ( load ser ) >> inSession' ( addFiles (rc_priority conf) paths ) ) ) defaultState
        return ()
      Query -> error "Not implemented"
      Backup dest ->
        flip evalStateT defaultState $ do
          onErrorDie ( load ser )
          inSession' ( withKnowledge kd >> ( autoBackup dest $ fromMaybe 1000000 $ rc_maxsize conf ) )
      Restore ->
        case paths of
          [sid, src, dest] -> evalStateT ( withKnowledge kd >> ( inSession' $ onErrorDie $ restore True sid src dest ) >> return () ) defaultState
          _ -> error "Usage: restore srcstor srcpath destpath"
      Merge ->
        case reverse paths of
          dest : srcs -> mergeDBs ( reverse srcs ) dest defaultState
          _ -> error "Usage: merge DBfile ... destDB"
      CheckBackup b -> noExtra $ do
        checkBackup ( not $ rc_rescan conf ) b
      Interact ->
        noExtra $ evalStateT ( onErrorDie initDB >> onErrorDie ( load ser ) >> runShell env ) defaultState
      Test -> liftIO $ runTest paths

  -- Execute action in appropriate monad
  if rc_log conf then
    flip evalStateT Nothing $ deLoggingIO $ runIOWithBuffer exec
    else
    runIOWithBuffer exec
